from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from cythereal_magic.api.admin_api import AdminApi
from cythereal_magic.api.cythereal_magic_api import CytherealMagicApi
